@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 """
 Converts ("coins") datasets in the sourcefolder to nifti / json / tsv datasets in the
 bidsfolder according to the BIDS standard. Check and edit the bidsmap.yaml file to
@@ -6,7 +6,10 @@ your needs using the bidseditor.py tool before running this function. You can ru
 bidscoiner.py after all data is collected, or run / re-run it whenever new data has
 been added to the source folder (presuming the scan protocol hasn't changed). If you
 delete a (subject/) session folder from the bidsfolder, it will be re-created from the
-sourcefolder the next time you run the bidscoiner.
+sourcefolder the next time you run the bidscoiner. Image tags indicating properties
+such as echo-number or complex data can be appended to the "acq" value if the BIDS
+datatype does not provide for this (e.g. "sub-01_acq-MEMPRAGE_T1w.nii" becomes
+"sub-01_acq-MEMPRAGEe1_T1w.nii")
 
 Provenance information, warnings and error messages are stored in the
 bidsfolder/code/bidscoin/bidscoiner.log file.
@@ -20,9 +23,10 @@ import logging
 import shutil
 from pathlib import Path
 try:
-    from bidscoin import bids
+    from bidscoin import bids, physio
 except ImportError:
     import bids         # This should work if bidscoin was not pip-installed
+    import physio
 
 LOGGER = logging.getLogger('bidscoin')
 
@@ -32,6 +36,7 @@ def coin_data2bids(dataformat: str, session: Path, bidsmap: dict, bidsfolder: Pa
     Converts the session source-files into BIDS-valid nifti-files in the corresponding bidsfolder and
     extracts personals (e.g. Age, Sex) from the source header
 
+    :param dataformat:  The format of the raw input data that is to be coined (e.g. 'DICOM' or 'PAR', see bids.get_dataformat)
     :param session:     The full-path name of the subject/session source file/folder
     :param bidsmap:     The full mapping heuristics from the bidsmap YAML-file
     :param bidsfolder:  The full-path name of the BIDS root-folder
@@ -46,12 +51,14 @@ def coin_data2bids(dataformat: str, session: Path, bidsmap: dict, bidsfolder: Pa
         sourcefile = Path()
         sources    = bids.lsdirs(session)
         for source in sources:
-            sourcefile = bids.get_dicomfile(source)
+            sourcefile   = bids.get_dicomfile(source)
+            manufacturer = bids.get_dicomfield('Manufacturer', sourcefile)
             if sourcefile.name:
                 break
 
     elif dataformat=='PAR':
         sources = bids.get_parfiles(session)
+        manufacturer = 'Philips Medical Systems'
         if sources:
             sourcefile = sources[0]
 
@@ -96,109 +103,135 @@ def coin_data2bids(dataformat: str, session: Path, bidsmap: dict, bidsfolder: Pa
             continue
 
         # Get a matching run from the bidsmap
-        run, modality, index = bids.get_matching_run(sourcefile, bidsmap, dataformat)
+        run, datatype, index = bids.get_matching_run(sourcefile, bidsmap, dataformat)
 
         # Check if we should ignore this run
-        if modality == bids.ignoremodality:
+        if datatype == bids.ignoredatatype:
             LOGGER.info(f"Leaving out: {source}")
             continue
 
         # Check if we already know this run
         if index is None:
-            LOGGER.warning(f"Skipping unknown '{modality}': {sourcefile}\n-> re-run the bidsmapper and delete {session} to solve this warning")
+            LOGGER.error(f"Skipping unknown '{datatype}' run: {sourcefile}\n-> Re-run the bidsmapper and delete {bidsses} to solve this warning")
             continue
 
         LOGGER.info(f"Processing: {source}")
 
-        # Create the BIDS session/modality folder
-        bidsmodality = bidsses/modality
-        bidsmodality.mkdir(parents=True, exist_ok=True)
+        # Create the BIDS session/datatype output folder
+        if run['bids']['suffix'] in bids.get_derivatives(datatype):
+            outfolder = bidsfolder/'derivatives'/manufacturer.replace(' ','')/subid/sesid/datatype
+        else:
+            outfolder = bidsses/datatype
+        outfolder.mkdir(parents=True, exist_ok=True)
 
         # Compose the BIDS filename using the matched run
-        bidsname = bids.get_bidsname(subid, sesid, modality, run)
-        runindex = run['bids']['run']
+        bidsname  = bids.get_bidsname(subid, sesid, run)
+        runindex  = run['bids'].get('run', '')
         if runindex.startswith('<<') and runindex.endswith('>>'):
-            bidsname = bids.increment_runindex(bidsmodality, bidsname)
+            bidsname = bids.increment_runindex(outfolder, bidsname)
+        jsonfiles = [(outfolder/bidsname).with_suffix('.json')]      # List -> Collect the associated json-files (for updating them later) -- possibly > 1
 
         # Check if file already exists (-> e.g. when a static runindex is used)
-        if (bidsmodality/bidsname).with_suffix('.json').is_file():
-            LOGGER.warning(f"{bidsmodality/bidsname}.* already exists -- check your results carefully!")
+        if (outfolder/bidsname).with_suffix('.json').is_file():
+            LOGGER.warning(f"{outfolder/bidsname}.* already exists and will be deleted -- check your results carefully!")
+            for ext in ('.nii.gz', '.nii', '.json', '.bval', '.bvec', 'tsv.gz'):
+                (outfolder/bidsname).with_suffix(ext).unlink(missing_ok=True)
+
+        # Convert physiological log files (dcm2niix can't handle these)
+        if run['bids']['suffix'] == 'physio':
+            if bids.get_dicomfile(source, 2).name:
+                LOGGER.warning(f"Found > 1 DICOM file in {source}, using: {sourcefile}")
+            physiodata = physio.readphysio(sourcefile)
+            physio.physio2tsv(physiodata, outfolder/bidsname)
 
         # Convert the source-files in the run folder to nifti's in the BIDS-folder
-        command = '{path}dcm2niix {args} -f "{filename}" -o "{outfolder}" "{source}"'.format(
-            path      = bidsmap['Options']['dcm2niix']['path'],
-            args      = bidsmap['Options']['dcm2niix']['args'],
-            filename  = bidsname,
-            outfolder = bidsmodality,
-            source    = source)
-        if not bids.run_command(command):
-            continue
+        else:
+            command = '{path}dcm2niix {args} -f "{filename}" -o "{outfolder}" "{source}"'.format(
+                path      = bidsmap['Options']['dcm2niix']['path'],
+                args      = bidsmap['Options']['dcm2niix']['args'],
+                filename  = bidsname,
+                outfolder = outfolder,
+                source    = source)
+            if not bids.run_command(command):
+                continue
 
-        # Replace uncropped output image with the cropped one
-        if '-x y' in bidsmap['Options']['dcm2niix']['args']:
-            for filename in sorted(bidsmodality.glob(bidsname + '*_Crop_*')):                                   # e.g. *_Crop_1.nii.gz
-                ext         = ''.join(filename.suffixes)
-                newfilename = str(filename).rsplit(ext,1)[0].rsplit('_Crop_',1)[0] + ext
-                LOGGER.info(f"Found dcm2niix _Crop_ suffix, replacing original file\n{filename} ->\n{newfilename}")
-                filename.replace(newfilename)
+            # Replace uncropped output image with the cropped one
+            if '-x y' in bidsmap['Options']['dcm2niix']['args']:
+                for dcm2niixfile in sorted(outfolder.glob(bidsname + '*_Crop_*')):                              # e.g. *_Crop_1.nii.gz
+                    ext         = ''.join(dcm2niixfile.suffixes)
+                    newbidsfile = str(dcm2niixfile).rsplit(ext,1)[0].rsplit('_Crop_',1)[0] + ext
+                    LOGGER.info(f"Found dcm2niix _Crop_ postfix, replacing original file\n{dcm2niixfile} ->\n{newbidsfile}")
+                    dcm2niixfile.replace(newbidsfile)
 
-        # Rename all files ending with _c%d, _e%d and _ph (and any combination of these) that are added by dcm2niix for multi-coil data, multi-echo data and phase data
-        # See: https://github.com/rordenlab/dcm2niix/blob/master/FILENAMING.md
-        jsonfiles = []                                                                                          # Collect the associated json-files (for updating them later) -- possibly > 1
-        for dcm2niisuffix in ('_c', '_e', '_ph', '_i'):
-            for filename in sorted(bidsmodality.glob(f"{bidsname}*{dcm2niisuffix}*")):
-                ext             = ''.join(filename.suffixes)
-                basepath, index = str(filename).rsplit(ext)[0].rsplit(dcm2niisuffix,1)                          # basepath = the name without the added stuff (i.e. bidsmodality/bidsname), index = added dcm2niix index (e.g. _c1 -> index=1)
-                basesuffix      = basepath.rsplit('_',1)[1]                                                     # The BIDS suffix, e.g. basepath = *_magnitude1 -> basesuffix=magnitude1
-                index           = index.split('_')[0].zfill(2)                                                  # Zero padd as specified in the BIDS-standard (assuming two digits is sufficient); strip following suffices (fieldmaps produce *_e2_ph files)
+            # Rename all files that got additional postfixes from dcm2niix. See: https://github.com/rordenlab/dcm2niix/blob/master/FILENAMING.md
+            dcm2niixpostfixes = ('_c', '_i', '_Eq', '_real', '_imaginary', '_MoCo', '_t', '_Tilt', '_e', '_ph')
+            dcm2niixfiles     = sorted(set([dcm2niixfile for dcm2niixpostfix in dcm2niixpostfixes for dcm2niixfile in outfolder.glob(f"{bidsname}*{dcm2niixpostfix}*")]))
+            for dcm2niixfile in dcm2niixfiles:
+                ext         = ''.join(dcm2niixfile.suffixes)
+                postfixes   = str(dcm2niixfile).split(bidsname)[1].rsplit(ext)[0].split('_')[1:]
+                newbidsname = dcm2niixfile.name                                                                 # Strip the additional postfixes and assign them to bids entities in the for-loop below
+                for postfix in postfixes:                                                                       # dcm2niix postfixes _c%d, _e%d and _ph (and any combination of these in that order) are for multi-coil data, multi-echo data and phase data
 
-                # Phase data may be stored in the magnitude data source (e.g. Philips fieldmaps)
-                if 'ph' in filename.name.rsplit(ext)[0].split('_'):
-                    basepath = basepath.replace('_magnitude', '_phase')
+                    # Patch the echo entity in the newbidsname with the dcm2niix echo info                      # NB: We can't rely on the bids-entity info here because manufacturers can e.g. put multiple echos in one series / run-folder
+                    if postfix[0]=='e' and bids.get_bidsvalue(newbidsname, 'echo'):                             # NB: Check if postfix[0]=='e' uniquely refers to the right dcm2niixpostfix
+                        echonr = f"_{postfix}"                                                                  # E.g. echonr='_e1' or echonr='_pha'
+                        for dcm2niixpostfix in dcm2niixpostfixes:
+                            echonr = echonr.replace(dcm2niixpostfix,'')                                         # Strip the dcm2niixpostfix to keep the echonr info. E.g. [echonr='_e1' or echonr='_pha'] -> [echonr='1' or echonr='a']
+                        if echonr.isalpha():
+                            echonr = ord(echonr) - 95                                                           # dcm2niix adds an alphabetically ordered character if it outputs more than one image with the same name. Convert character to echo-number: '' -> 1, 'a'->2, etc
+                        elif not echonr:
+                            echonr = 1
+                        newbidsname = bids.get_bidsvalue(newbidsname, 'echo', str(echonr))                      # In contrast to other labels, run and echo labels MUST be integers. Those labels MAY include zero padding, but this is NOT RECOMMENDED to maintain their uniqueness
 
-                # This is a special hack: dcm2niix does not always add a _c/_e suffix for the first(?) coil/echo image -> add it when we encounter a **_e2/_c2 file
-                # https://github.com/rordenlab/dcm2niix/issues/381
-                if dcm2niisuffix in ('_c','_e') and int(index)==2 and basesuffix not in ['magnitude1', 'phase1']:    # For fieldmaps: *_magnitude1_e[index] -> *_magnitude[index] (This is handled below)
-                    filename_ce = Path(basepath + ext)                                                          # The file without the _c1/_e1 suffix
-                    if dcm2niisuffix=='_e' and bids.get_bidsvalue(basepath, 'echo'):
-                        newbasepath_ce = Path(bids.get_bidsvalue(basepath, 'echo', '1'))
+                    # Patch fieldmap images (NB: datatype=='fmap' is too broad, see the fmap.yaml file)
+                    elif run['bids']['suffix'] in ('magnitude','magnitude1','magnitude2','phase1','phase2','phasediff','fieldmap'):
+                        if len(dcm2niixfiles) not in (0, 2, 4, 6, 8):                                           # Phase / echo data may be stored in the same data source / run folder
+                            LOGGER.warning(f"Unknown fieldmap {outfolder/bidsname} for '{postfix}'")
+                        newbidsname = newbidsname.replace('_fieldmap_ph',    '_fieldmap')
+                        newbidsname = newbidsname.replace('_magnitude_e1',   '_magnitude')
+                        newbidsname = newbidsname.replace('_magnitude_ph',   '_fieldmap')
+                        newbidsname = newbidsname.replace('_magnitude1_e1',  '_magnitude1')
+                        newbidsname = newbidsname.replace('_magnitude2_e1',  '_magnitude1')                     # This can happen when the e2 image is stored in the same directory as the e1 image, but with the e2 listed first
+                        newbidsname = newbidsname.replace('_magnitude1_e2',  '_magnitude2')
+                        if len(dcm2niixfiles) == 8:
+                            newbidsname = newbidsname.replace('_magnitude1_ph',  '_phase1')                     # Two magnitude + 2 phase images in one folder / datasource
+                        else:
+                            newbidsname = newbidsname.replace('_magnitude1_ph',  '_phasediff')                  # One or two magnitude + 1 phasediff image
+                        newbidsname = newbidsname.replace('_magnitude1a',    '_magnitude2')
+                        newbidsname = newbidsname.replace('_magnitude1_pha', '_phase2')
+                        newbidsname = newbidsname.replace('_magnitude2_e2',  '_magnitude2')
+                        newbidsname = newbidsname.replace('_magnitude2_ph',  '_phase2')
+                        newbidsname = newbidsname.replace('_phase1_e1',      '_phase1')
+                        newbidsname = newbidsname.replace('_phase2_e1',      '_phase1')                         # This can happen when the e2 image is stored in the same directory as the e1 image, but with the e2 listed first
+                        newbidsname = newbidsname.replace('_phase1_ph',      '_phase1')
+                        newbidsname = newbidsname.replace('_phase1_e2',      '_phase2')
+                        newbidsname = newbidsname.replace('_phase2_e2',      '_phase2')
+                        newbidsname = newbidsname.replace('_phase2_ph',      '_phase2')
+
+                    # Append the dcm2niix info to acq-label, may need to be improved / elaborated for future BIDS standards, supporting multi-coil data
                     else:
-                        newbasepath_ce = Path(bids.get_bidsvalue(basepath, 'dummy', dcm2niisuffix.upper()[1:] + '1'.zfill(len(index))))  # --> append to acq-label, may need to be elaborated for future BIDS standards, supporting multi-coil data
-                    newfilename_ce = newbasepath_ce.with_suffix(ext)                                            # The file as it should have been
-                    if filename_ce.is_file():
-                        if filename_ce != newfilename_ce:
-                            LOGGER.warning(f"Found no dcm2niix {dcm2niisuffix} suffix for image instance 1, renaming\n{filename_ce} ->\n{newfilename_ce}\nConsider upgrading dcm2niix: https://github.com/rordenlab/dcm2niix/issues/381")
-                            filename_ce.replace(newfilename_ce)
-                        if ext == '.json':
-                            jsonfiles.append(newbasepath_ce.with_suffix('.json'))
+                        newbidsname = bids.get_bidsvalue(newbidsname, 'dummy', postfix)
 
-                # Patch the basepath with the dcm2niix suffix info (we can't rely on the basepath info here because Siemens can e.g. put multiple echos in one series / run-folder)
-                if dcm2niisuffix=='_e' and bids.get_bidsvalue(basepath, 'echo') and index:
-                    basepath = bids.get_bidsvalue(basepath, 'echo', str(int(index)))                            # In contrast to other labels, run and echo labels MUST be integers. Those labels MAY include zero padding, but this is NOT RECOMMENDED to maintain their uniqueness
-
-                elif dcm2niisuffix=='_e' and basesuffix in ('magnitude1','magnitude2','phase1','phase2') and index:  # i.e. modality == 'fmap'
-                    basepath = basepath[0:-1] + str(int(index))                                                 # basepath: *_magnitude1_e[index] -> *_magnitude[index] and *_phase1_e[index]_ph -> *_phase[index]
-
-                elif dcm2niisuffix=='_e' and basesuffix=='phasediff' and index:                                 # i.e. modality == 'fmap'
-                    pass
-
-                else:
-                    basepath = bids.get_bidsvalue(basepath, 'dummy', dcm2niisuffix.upper()[1:] + index)             # --> append to acq-label, may need to be elaborated for future BIDS standards, supporting multi-coil data
+                    # Remove the added postfix from the new bidsname
+                    newbidsname = newbidsname.replace(f"_{postfix}_",'_')                                       # If it is not last
+                    newbidsname = newbidsname.replace(f"_{postfix}.",'.')                                       # If it is last
 
                 # Save the file with a new name
-                newbidsname = str(Path(basepath).name)
                 if runindex.startswith('<<') and runindex.endswith('>>'):
-                    newbidsname = bids.increment_runindex(bidsmodality, newbidsname, ext)                       # Update the runindex now that the acq-label has changed
-                newfilename = (bidsmodality/newbidsname).with_suffix(ext)
-                LOGGER.info(f"Found dcm2niix {dcm2niisuffix} suffix, renaming\n{filename} ->\n{newfilename}")
-                filename.replace(newfilename)
+                    newbidsname = bids.increment_runindex(outfolder, newbidsname, '')                           # Update the runindex now that the acq-label has changed
+                newbidsfile = outfolder/newbidsname
+                LOGGER.info(f"Found dcm2niix {postfixes} postfixes, renaming\n{dcm2niixfile} ->\n{newbidsfile}")
+                if newbidsfile.is_file():
+                    LOGGER.warning(f"Overwriting existing {newbidsfile} file -- check your results carefully!")
+                dcm2niixfile.replace(newbidsfile)
                 if ext == '.json':
-                    jsonfiles.append((bidsmodality/newbidsname).with_suffix('.json'))
+                    oldjsonfile = (outfolder/bidsname).with_suffix('.json')
+                    if oldjsonfile in jsonfiles and not oldjsonfile.is_file():
+                        jsonfiles.remove((outfolder/bidsname).with_suffix('.json'))
+                    jsonfiles.append(newbidsfile)
 
         # Loop over and adapt all the newly produced json files and write to the scans.tsv file (every nifti-file comes with a json-file)
-        if not jsonfiles:
-            jsonfiles = [(bidsmodality/bidsname).with_suffix('.json')]
-        for jsonfile in set(jsonfiles):
+        for jsonfile in sorted(set(jsonfiles)):
 
             # Check if dcm2niix behaved as expected
             if not jsonfile.is_file():
@@ -206,7 +239,7 @@ def coin_data2bids(dataformat: str, session: Path, bidsmap: dict, bidsfolder: Pa
                 continue
 
             # Add a dummy b0 bval- and bvec-file for any file without a bval/bvec file (e.g. sbref, b0 scans)
-            if modality == 'dwi':
+            if datatype == 'dwi':
                 bvecfile = jsonfile.with_suffix('.bvec')
                 bvalfile = jsonfile.with_suffix('.bval')
                 if not bvecfile.is_file():
@@ -219,7 +252,7 @@ def coin_data2bids(dataformat: str, session: Path, bidsmap: dict, bidsfolder: Pa
                         bval_fid.write('0\n')
 
             # Add the TaskName to the func json-file
-            elif modality == 'func':
+            elif datatype == 'func':
                 with jsonfile.open('r') as json_fid:
                     data = json.load(json_fid)
                 if not 'TaskName' in data:
@@ -229,25 +262,31 @@ def coin_data2bids(dataformat: str, session: Path, bidsmap: dict, bidsfolder: Pa
                         json.dump(data, json_fid, indent=4)
 
             # Parse the acquisition time from the json file or else from the source header (NB: assuming the source file represents the first acquisition)
-            with jsonfile.open('r') as json_fid:
-                data = json.load(json_fid)
-            if 'AcquisitionTime' not in data or not data['AcquisitionTime']:
-                data['AcquisitionTime'] = bids.get_sourcefield('AcquisitionTime', sourcefile)       # DICOM
-            if not data['AcquisitionTime']:
-                data['AcquisitionTime'] = bids.get_sourcefield('exam_date', sourcefile)             # PAR/XML
-            acq_time = dateutil.parser.parse(data['AcquisitionTime'])
-            scanpath = list(jsonfile.parent.glob(jsonfile.stem + '.nii*'))[0].relative_to(bidsses)  # Find the corresponding nifti file (there should be only one, let's not make assumptions about the .gz extension)
-            scans_table.loc[scanpath.as_posix(), 'acq_time'] = '1925-01-01T' + acq_time.strftime('%H:%M:%S')
+            niifile = list(jsonfile.parent.glob(jsonfile.stem + '.nii*'))       # Find the corresponding nifti file (there should be only one, let's not make assumptions about the .gz extension)
+            if niifile and datatype not in bidsmap['Options']['bidscoin']['bidsignore'] and not run['bids']['suffix'] in bids.get_derivatives(datatype):
+                with jsonfile.open('r') as json_fid:
+                    data = json.load(json_fid)
+                if 'AcquisitionTime' not in data or not data['AcquisitionTime']:
+                    data['AcquisitionTime'] = bids.get_sourcefield('AcquisitionTime', sourcefile)       # DICOM
+                if not data['AcquisitionTime']:
+                    data['AcquisitionTime'] = bids.get_sourcefield('exam_date', sourcefile)             # PAR/XML
+                try:
+                    acq_time = dateutil.parser.parse(data['AcquisitionTime'])
+                except:
+                    LOGGER.warning(f"Could not parse the acquisition time from: '{data['AcquisitionTime']}' in {sourcefile}")
+                    acq_time = dateutil.parser.parse('00:00:00')
+                scanpath = niifile[0].relative_to(bidsses)
+                scans_table.loc[scanpath.as_posix(), 'acq_time'] = '1925-01-01T' + acq_time.strftime('%H:%M:%S')
 
     # Write the scans_table to disk
     LOGGER.info(f"Writing acquisition time data to: {scans_tsv}")
     scans_table.sort_values(by=['acq_time','filename'], inplace=True)
     scans_table.to_csv(scans_tsv, sep='\t', encoding='utf-8')
 
-    # Add IntendedFor and TE1+TE2 meta-data the fieldmap json-files. This has been postponed untill all modalities have been processed (i.e. so that all target images are indeed on disk)
+    # Add IntendedFor and TE1+TE2 meta-data to the fieldmap json-files. This has been postponed untill all datatypes have been processed (i.e. so that all target images are indeed on disk)
     if bidsmap[dataformat]['fmap'] is not None:
         for fieldmap in bidsmap[dataformat]['fmap']:
-            bidsname    = bids.get_bidsname(subid, sesid, 'fmap', fieldmap)
+            bidsname    = bids.get_bidsname(subid, sesid, fieldmap)
             niifiles    = []
             intendedfor = fieldmap['bids']['IntendedFor']
 
@@ -259,11 +298,11 @@ def coin_data2bids(dataformat: str, session: Path, bidsmap: dict, bidsfolder: Pa
                     intendedfor = [intendedfor]
                 for selector in intendedfor:
                     niifiles.extend([Path(niifile).relative_to(bidsfolder/subid)
-                                     for niifile in sorted(bidsses.rglob(f"*{selector}*.nii*")) if selector])                                   # Search in all runs using a relative path to the subject folder
+                                     for niifile in sorted(bidsses.rglob(f"*{selector}*.nii*")) if selector])       # Search in all runs using a relative path to the subject folder
             else:
                 intendedfor = []
 
-            # Get the set of json-files (account for multiple runs in one data source and dcm2niix suffixes inserted into the acquisition label)
+            # Get the set of json-files (account for multiple runs in one data source and dcm2niix postfixes inserted into the acquisition label)
             jsonfiles = []
             acqlabel  = bids.get_bidsvalue(bidsname, 'acq')
             patterns  = (bidsname.replace('_run-1_',     '_run-[0-9]*_').
@@ -281,7 +320,7 @@ def coin_data2bids(dataformat: str, session: Path, bidsmap: dict, bidsfolder: Pa
                     jsonfiles.extend(list((bidsses/'fmap').glob(cepattern + '.json')))
 
             # Save the meta-data in the jsonfiles
-            for jsonfile in set(jsonfiles):
+            for jsonfile in sorted(set(jsonfiles)):
 
                 # Add the IntendedFor data
                 with jsonfile.open('r') as json_fid:
@@ -293,7 +332,7 @@ def coin_data2bids(dataformat: str, session: Path, bidsmap: dict, bidsfolder: Pa
                         LOGGER.warning(f"Empty 'IntendedFor' fieldmap value in {jsonfile}: the search for {intendedfor} gave no results")
                     else:
                         LOGGER.warning(f"Empty 'IntendedFor' fieldmap value in {jsonfile}: the IntendedFor value of the bidsmap entry was empty")
-                    data['IntendedFor'] = [niifile.as_posix() for niifile in niifiles]                                                              # The path needs to use forward slashes instead of backward slashes
+                    data['IntendedFor'] = [niifile.as_posix() for niifile in niifiles]      # The path needs to use forward slashes instead of backward slashes
                     with jsonfile.open('w') as json_fid:
                         json.dump(data, json_fid, indent=4)
 
@@ -302,7 +341,7 @@ def coin_data2bids(dataformat: str, session: Path, bidsmap: dict, bidsfolder: Pa
                     json_magnitude = [None, None]
                     TE             = [None, None]
                     for n in (0,1):
-                        json_magnitude[n] = jsonfile.parent / jsonfile.name.replace('_phasediff', f"_magnitude{n+1}")
+                        json_magnitude[n] = jsonfile.parent/jsonfile.name.replace('_phasediff', f"_magnitude{n+1}")
                         if not json_magnitude[n].is_file():
                             LOGGER.error(f"Could not find expected magnitude{n+1} image associated with: {jsonfile}")
                         else:
@@ -329,7 +368,7 @@ def coin_data2bids(dataformat: str, session: Path, bidsmap: dict, bidsfolder: Pa
             if 'session_id' not in personals:
                 personals['session_id'] = sesid
             else:
-                return                                              # Only from the first session -> BIDS specification
+                return                                              # Only take data from the first session -> BIDS specification
         age = bids.get_dicomfield('PatientAge', sourcefile)         # A string of characters with one of the following formats: nnnD, nnnW, nnnM, nnnY
         if age.endswith('D'):
             personals['age'] = str(int(float(age.rstrip('D'))/365.2524))
@@ -396,7 +435,7 @@ def coin_plugin(session: Path, bidsmap: dict, bidsfolder: Path, personals: dict)
             module.bidscoiner_plugin(session, bidsmap, bidsfolder, personals)
 
 
-def bidscoiner(rawfolder: str, bidsfolder: str, subjects: list=[], force: bool=False, participants: bool=False, bidsmapfile: str='bidsmap.yaml', subprefix: str='sub-', sesprefix: str='ses-') -> None:
+def bidscoiner(rawfolder: str, bidsfolder: str, subjects: list=(), force: bool=False, participants: bool=False, bidsmapfile: str='bidsmap.yaml', subprefix: str='sub-', sesprefix: str='ses-') -> None:
     """
     Main function that processes all the subjects and session in the sourcefolder and uses the
     bidsmap.yaml file in bidsfolder/code/bidscoin to cast the data into the BIDS folder.
@@ -431,12 +470,14 @@ def bidscoiner(rawfolder: str, bidsfolder: str, subjects: list=[], force: bool=F
     dataset_file = bidsfolder/'dataset_description.json'
     if not dataset_file.is_file():
         dataset_description = {"Name":                  "REQUIRED. Name of the dataset",
-                               "BIDSVersion":           bids.bidsversion(),
-                               "License":               "RECOMMENDED. What license is this dataset distributed under?. The use of license name abbreviations is suggested for specifying a license",
+                               "BIDSVersion":           str(bids.bidsversion()),
+                               "DatasetType":           "raw",
+                               "License":               "RECOMMENDED. The license for the dataset. The use of license name abbreviations is RECOMMENDED for specifying a license. The corresponding full license text MAY be specified in an additional LICENSE file",
                                "Authors":               ["OPTIONAL. List of individuals who contributed to the creation/curation of the dataset"],
-                               "Acknowledgements":      "OPTIONAL. List of individuals who contributed to the creation/curation of the dataset",
+                               "Acknowledgements":      "OPTIONAL. Text acknowledging contributions of individuals or institutions beyond those listed in Authors or Funding",
                                "HowToAcknowledge":      "OPTIONAL. Instructions how researchers using this dataset should acknowledge the original authors. This field can also be used to define a publication that should be cited in publications that use the dataset",
                                "Funding":               ["OPTIONAL. List of sources of funding (grant numbers)"],
+                               "EthicsApprovals":    	["OPTIONAL. List of ethics committee approvals of the research protocols and/or protocol identifiers"],
                                "ReferencesAndLinks":    ["OPTIONAL. List of references to publication that contain information on the dataset, or links", "https://github.com/Donders-Institute/bidscoin"],
                                "DatasetDOI":            "OPTIONAL. The Document Object Identifier of the dataset (not the corresponding paper)"}
         LOGGER.info(f"Creating dataset description file: {dataset_file}")
@@ -479,7 +520,7 @@ def bidscoiner(rawfolder: str, bidsfolder: str, subjects: list=[], force: bool=F
         with participants_json.open('r') as json_fid:
             participants_dict = json.load(json_fid)
     else:
-        participants_dict = dict()
+        participants_dict = {'participant_id': {'Description': 'Unique participant identifier'}}
 
     # Get the list of subjects
     if not subjects:
@@ -487,7 +528,7 @@ def bidscoiner(rawfolder: str, bidsfolder: str, subjects: list=[], force: bool=F
         if not subjects:
             LOGGER.warning(f"No subjects found in: {rawfolder/subprefix}*")
     else:
-        subjects = [subprefix + re.sub(f"^{subprefix}", '', subject) for subject in subjects]        # Make sure there is a "sub-" prefix
+        subjects = [subprefix + re.sub(f"^{subprefix}", '', subject) for subject in subjects]   # Make sure there is a "sub-" prefix
         subjects = [rawfolder/subject for subject in subjects if (rawfolder/subject).is_dir()]
 
     # Loop over all subjects and sessions and convert them using the bidsmap entries
@@ -505,8 +546,7 @@ def bidscoiner(rawfolder: str, bidsfolder: str, subjects: list=[], force: bool=F
         for session in sessions:
 
             # Unpack the data in a temporary folder if it is tarballed/zipped and/or contains a DICOMDIR file
-            bidssession       = bidsfolder/session.relative_to(rawfolder)  # Append the sub-*/ses-* subdirectories from the rawfolder to the bidsfolder
-            session, unpacked = bids.unpack(session, subprefix, sesprefix, '*')
+            session, unpacked = bids.unpack(session, subprefix, sesprefix)
 
             # See what dataformat we have
             dataformat = bids.get_dataformat(session)
@@ -516,19 +556,21 @@ def bidscoiner(rawfolder: str, bidsfolder: str, subjects: list=[], force: bool=F
 
             # Check if we should skip the session-folder
             if not force:
+                subid, sesid = bids.get_subid_sesid(session/'dum.my', subprefix=subprefix, sesprefix=sesprefix)
+                bidssession  = bidsfolder/subid/sesid
                 if not bidsmap[dataformat]['session']:
                     bidssession = bidssession.parent
-                modalities = []
-                for modality in bids.lsdirs(bidssession):                                   # See what modalities we already have in the bids session-folder
-                    if modality.glob('*') and bidsmap[dataformat].get(modality.name):       # See if we are going to add data for this modality
-                        modalities.append(modality.name)
-                if modalities:
-                    LOGGER.info(f"Skipping processed session: {bidssession} already has {modalities} data (use the -f option to overrule)")
+                datatypes = []
+                for datatype in bids.lsdirs(bidssession):                               # See what datatypes we already have in the bids session-folder
+                    if datatype.glob('*') and bidsmap[dataformat].get(datatype.name):   # See if we are going to add data for this datatype
+                        datatypes.append(datatype.name)
+                if datatypes:
+                    LOGGER.info(f"Skipping processed session: {bidssession} already has {datatypes} data (use the -f option to overrule)")
                     continue
 
             LOGGER.info(f"Coining session: {session}")
 
-            # Update / append the sourde data mapping
+            # Update / append the source data mapping
             if dataformat in ('DICOM', 'PAR'):
                 coin_data2bids(dataformat, session, bidsmap, bidsfolder, personals, subprefix, sesprefix)
 
@@ -567,8 +609,7 @@ def bidscoiner(rawfolder: str, bidsfolder: str, subjects: list=[], force: bool=F
                 participants_dict[key] = dict(LongName     = 'Long (unabbreviated) name of the column',
                                               Description  = 'Description of the the column',
                                               Levels       = dict(Key='Value (This is for categorical variables: a dictionary of possible values (keys) and their descriptions (values))'),
-                                              Units        = 'Measurement units. [<prefix symbol>]<unit symbol> format following the SI standard is RECOMMENDED',
-                                              TermURL      = 'URL pointing to a formal definition of this type of data in an ontology available on the web')
+                                              Units        = 'Measurement units. [<prefix symbol>]<unit symbol> format following the SI standard is RECOMMENDED')
             participants_table.loc[personals['participant_id'], key] = personals[key]
 
     # Write the collected data to the participant files
