@@ -1,14 +1,10 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 """
-Creates a bidsmap.yaml YAML file in the bidsfolder/code/bidscoin that maps the information
-from all raw source data to the BIDS labels. You can check and edit the bidsmap file with
-the bidseditor (but also with any text-editor) before passing it to the bidscoiner. See the
-bidseditor help for more information and useful tips for running the bidsmapper in interactive
-mode (which is the default).
-
-N.B.: Institute users may want to use a site-customized template bidsmap (see the
---template option). The bidsmap_dccn template from the Donders Institute can serve as
-an example (or may even mostly work for other institutes out of the box).
+Creates a bidsmap.yaml YAML file in the bidsfolder/code/bidscoin that maps the
+information from all raw source data to the BIDS labels. You can check and edit
+the bidsmap file with the bidseditor (but also with any text-editor) before
+passing it to the bidscoiner. See the bidseditor help for more information and
+useful tips for running the bidsmapper in interactive mode (the default).
 """
 
 # Global imports (plugin modules may be imported when needed)
@@ -37,6 +33,7 @@ def build_bidsmap(dataformat: str, sourcefile: Path, bidsmap_new: dict, bidsmap_
     :param bidsmap_new: The bidsmap that we are building
     :param bidsmap_old: Full BIDS heuristics data structure, with all options, BIDS labels and attributes, etc
     :param template:    The bidsmap template with the default heuristics
+    :param store:       The paths of the source- and target-folder
     :param gui:         If not None, the user will not be asked for help if an unknown run is encountered
     :return:            The bidsmap with new entries in it
     """
@@ -47,11 +44,11 @@ def build_bidsmap(dataformat: str, sourcefile: Path, bidsmap_new: dict, bidsmap_
         return bidsmap_new
 
     # See if we can find a matching run in the old bidsmap
-    run, modality, index = bids.get_matching_run(sourcefile, bidsmap_old, dataformat)
+    run, datatype, index = bids.get_matching_run(sourcefile, bidsmap_old, dataformat)
 
     # If not, see if we can find a matching run in the template
     if index is None:
-        run, modality, _ = bids.get_matching_run(sourcefile, template, dataformat)
+        run, datatype, _ = bids.get_matching_run(sourcefile, template, dataformat)
 
     # See if we have collected the run in our new bidsmap
     if not bids.exist_run(bidsmap_new, dataformat, '', run):
@@ -64,16 +61,16 @@ def build_bidsmap(dataformat: str, sourcefile: Path, bidsmap_new: dict, bidsmap_
             run['provenance'] = str(sourcefile.resolve())
 
         # Communicate with the user if the run was not present in bidsmap_old or in template, i.e. that we found a new sample
-        LOGGER.info(f"Found '{modality}' {dataformat} sample: {sourcefile}")
+        LOGGER.info(f"Found '{datatype}' {dataformat} sample: {sourcefile}")
 
         # Copy the filled-in run over to the new bidsmap
-        bidsmap_new = bids.append_run(bidsmap_new, dataformat, modality, run)
+        bidsmap_new = bids.append_run(bidsmap_new, dataformat, datatype, run)
 
         # Launch a GUI to ask the user for help if the new run comes from the template (i.e. was not yet in the old bidsmap)
         if gui and gui.interactive==2 and index is None:
 
             # Open the interactive edit window to get the new mapping
-            dialog_edit = bidseditor.EditDialog(dataformat, sourcefile, modality, bidsmap_new, template, gui.subprefix, gui.sesprefix)
+            dialog_edit = bidseditor.EditDialog(dataformat, sourcefile, datatype, bidsmap_new, template, gui.subprefix, gui.sesprefix)
             dialog_edit.exec()
 
             # Get the result
@@ -108,8 +105,6 @@ def build_niftimap(session: Path, bidsmap_new: dict, bidsmap_old: dict) -> dict:
     if not session or not bidsmap_old['Nifti']:
         return bidsmap_new
 
-    # TODO: Loop through all bidsmodalities and runs
-
     return bidsmap_new
 
 
@@ -127,8 +122,6 @@ def build_filesystemmap(session: Path, bidsmap_new: dict, bidsmap_old: dict) ->
     if not session or not bidsmap_old['FileSystem']:
         return bidsmap_new
 
-    # TODO: Loop through all bidsmodalities and runs
-
     return bidsmap_new
 
 
@@ -180,10 +173,6 @@ def bidsmapper(rawfolder: str, bidsfolder: str, bidsmapfile: str, templatefile:
     bidsmapfile    = Path(bidsmapfile)
     templatefile   = Path(templatefile)
     bidscoinfolder = bidsfolder/'code'/'bidscoin'
-    if store:
-        store = dict(source=rawfolder, target=bidscoinfolder/'provenance')
-    else:
-        store = dict()
 
     # Start logging
     bids.setup_logging(bidscoinfolder/'bidsmapper.log')
@@ -196,15 +185,15 @@ def bidsmapper(rawfolder: str, bidsfolder: str, bidsmapfile: str, templatefile:
     bidsmap_old, _ = bids.load_bidsmap(bidsmapfile,  bidscoinfolder)
     template, _    = bids.load_bidsmap(templatefile, bidscoinfolder)
 
-    # Create the new bidsmap as a copy / bidsmap skeleton with no modality entries (i.e. bidsmap with empty lists)
+    # Create the new bidsmap as a copy / bidsmap skeleton with no datatype entries (i.e. bidsmap with empty lists)
     if bidsmap_old:
         bidsmap_new = copy.deepcopy(bidsmap_old)
     else:
         bidsmap_new = copy.deepcopy(template)
     for logic in ('DICOM', 'PAR', 'P7', 'Nifti', 'FileSystem'):
-        for modality in bids.bidsmodalities + (bids.unknownmodality, bids.ignoremodality):
-            if bidsmap_new[logic] and modality in bidsmap_new[logic]:
-                bidsmap_new[logic][modality] = None
+        for datatype in bids.bidsdatatypes + (bids.unknowndatatype, bids.ignoredatatype):
+            if bidsmap_new.get(logic) and datatype in bidsmap_new[logic]:
+                bidsmap_new[logic][datatype] = None
 
     # Start with an empty skeleton if we didn't have an old bidsmap
     if not bidsmap_old:
@@ -214,7 +203,7 @@ def bidsmapper(rawfolder: str, bidsfolder: str, bidsmapfile: str, templatefile:
     gui = interactive
     if gui:
         app = QApplication(sys.argv)
-        app.setApplicationName('BIDS editor')
+        app.setApplicationName(f"{bidsmapfile} - BIDS editor {bids.version()}")
         mainwin = bidseditor.MainWindow()
         gui = bidseditor.Ui_MainWindow()
         gui.interactive = interactive
@@ -225,7 +214,7 @@ def bidsmapper(rawfolder: str, bidsfolder: str, bidsmapfile: str, templatefile:
             QMessageBox.information(mainwin, 'BIDS mapping workflow',
                                     f"The bidsmapper will now scan {bidsfolder} and whenever "
                                     f"it detects a new type of scan it will ask you to identify it.\n\n"
-                                    f"It is important that you choose the correct BIDS modality "
+                                    f"It is important that you choose the correct BIDS datatype "
                                     f"(e.g. 'anat', 'dwi' or 'func') and suffix (e.g. 'bold' or 'sbref').\n\n"
                                     f"At the end you will be shown an overview of all the "
                                     f"different scan types and BIDScoin options (as in the "
@@ -245,9 +234,13 @@ def bidsmapper(rawfolder: str, bidsfolder: str, bidsmapfile: str, templatefile:
         for session in sessions:
 
             # Unpack the data in a temporary folder if it is tarballed/zipped and/or contains a DICOMDIR file
-            session, unpacked = bids.unpack(session, subprefix, sesprefix, '*')
+            session, unpacked = bids.unpack(session, subprefix, sesprefix)
             if unpacked:
                 store = dict(source=unpacked, target=bidscoinfolder/'provenance')
+            elif store:
+                store = dict(source=rawfolder, target=bidscoinfolder/'provenance')
+            else:
+                store = dict()
 
             # Loop of the different DICOM runs (series) and collect source files
             sourcefiles = []
@@ -290,30 +283,33 @@ def bidsmapper(rawfolder: str, bidsfolder: str, bidsmapfile: str, templatefile:
             if unpacked:
                 shutil.rmtree(session)
 
-    # Save the bidsmap in the bidscoinfolder
-    bidsmapfile = bidscoinfolder/'bidsmap.yaml'
-    bids.save_bidsmap(bidsmapfile, bidsmap_new)
-
-    # (Re)launch the bidseditor UI_MainWindow
     if not dataformat:
         LOGGER.warning('Could not determine the dataformat of the source data')
+
+    # (Re)launch the bidseditor UI_MainWindow
+    bidsmapfile = bidscoinfolder/'bidsmap.yaml'
     if gui:
         if not dataformat:
+            dataformat = 'DICOM'
             QMessageBox.information(mainwin, 'BIDS mapping workflow',
-                                    'Could not determine the dataformat of the source data.\n'
-                                    'You can try running the bidseditor tool yourself')
-        else:
-            QMessageBox.information(mainwin, 'BIDS mapping workflow',
-                                    f"The bidsmapper has finished scanning {rawfolder}\n\n"
-                                    f"Please carefully check all the different BIDS output names "
-                                    f"and BIDScoin options and (re)edit them to your needs.\n\n"
-                                    f"You can always redo this step later by re-running the "
-                                    f"bidsmapper or by just running the bidseditor tool")
+                                    'Could not determine the dataformat of the last\n'
+                                   f'source data run, defaulting to: {dataformat}\n'
+                                    'You can try running the bidseditor tool yourself if\n'
+                                    'this is not correct')
+        QMessageBox.information(mainwin, 'BIDS mapping workflow',
+                                f"The bidsmapper has finished scanning {rawfolder}\n\n"
+                                f"Please carefully check all the different BIDS output names "
+                                f"and BIDScoin options and (re)edit them to your needs.\n\n"
+                                f"You can always redo this step later by re-running the "
+                                f"bidsmapper or by just running the bidseditor tool")
 
-            LOGGER.info('Opening the bidseditor')
-            gui.setupUi(mainwin, bidsfolder, bidsmapfile, bidsmap_new, copy.deepcopy(bidsmap_new), template, dataformat, subprefix=subprefix, sesprefix=sesprefix)
-            mainwin.show()
-            app.exec()
+        LOGGER.info('Opening the bidseditor')
+        gui.setupUi(mainwin, bidsfolder, bidsmapfile, bidsmap_new, copy.deepcopy(bidsmap_new), template, dataformat, subprefix=subprefix, sesprefix=sesprefix)
+        mainwin.show()
+        app.exec()
+    else:
+        # Save the bidsmap in the bidscoinfolder
+        bids.save_bidsmap(bidsmapfile, bidsmap_new)
 
     LOGGER.info('-------------- FINISHED! -------------------')
     LOGGER.info('')
@@ -330,14 +326,14 @@ def main():
                                      description=textwrap.dedent(__doc__),
                                      epilog='examples:\n'
                                             '  bidsmapper /project/foo/raw /project/foo/bids\n'
-                                            '  bidsmapper /project/foo/raw /project/foo/bids -t bidsmap_dccn\n ')
+                                            '  bidsmapper /project/foo/raw /project/foo/bids -t bidsmap_template\n ')
     parser.add_argument('sourcefolder',       help='The study root folder containing the raw data in sub-#/[ses-#/]data subfolders (or specify --subprefix and --sesprefix for different prefixes)')
     parser.add_argument('bidsfolder',         help='The destination folder with the (future) bids data and the bidsfolder/code/bidscoin/bidsmap.yaml output file')
     parser.add_argument('-b','--bidsmap',     help='The bidsmap YAML-file with the study heuristics. If the bidsmap filename is relative (i.e. no "/" in the name) then it is assumed to be located in bidsfolder/code/bidscoin. Default: bidsmap.yaml', default='bidsmap.yaml')
-    parser.add_argument('-t','--template',    help='The bidsmap template with the default heuristics (this could be provided by your institute). If the bidsmap filename is relative (i.e. no "/" in the name) then it is assumed to be located in bidsfolder/code/bidscoin. Default: bidsmap_template.yaml', default='bidsmap_template.yaml')
+    parser.add_argument('-t','--template',    help='The bidsmap template with the default heuristics (this could be provided by your institute). If the bidsmap filename is relative (i.e. no "/" in the name) then it is assumed to be located in bidsfolder/code/bidscoin. Default: bidsmap_dccn.yaml', default='bidsmap_dccn.yaml')
     parser.add_argument('-n','--subprefix',   help="The prefix common for all the source subject-folders. Default: 'sub-'", default='sub-')
     parser.add_argument('-m','--sesprefix',   help="The prefix common for all the source session-folders. Default: 'ses-'", default='ses-')
-    parser.add_argument('-s','--store',       help="Flag to store the provenance data samples in the bidsfolder/'code'/'provenance' folder", action='store_true')
+    parser.add_argument('-s','--store',       help="Flag to store provenance data samples in the bidsfolder/'code'/'provenance' folder", action='store_true')
     parser.add_argument('-i','--interactive', help='{0}: The sourcefolder is scanned for different kinds of scans without any user interaction. {1}: The sourcefolder is scanned for different kinds of scans and, when finished, the resulting bidsmap is opened using the bidseditor. {2}: As {1}, except that already during scanning the user is asked for help if a new and unknown run is encountered. This option is most useful when re-running the bidsmapper (e.g. when the scan protocol was changed since last running the bidsmapper). Default: 1', type=int, choices=[0,1,2], default=1)
     parser.add_argument('-v','--version',     help='Show the BIDS and BIDScoin version', action='version', version=f'BIDS-version:\t\t{bids.bidsversion()}\nBIDScoin-version:\t{bids.version()}')
     args = parser.parse_args()
